/**
 * Copyright 2005 Jason LaDere
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Originally based on Project64 code.
 *
 */

/*
 * BasicKeyboardPlugin.java
 *
 * Created on January 1, 2006, 1:31 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package basiccontroller;

import java.awt.event.KeyEvent;
import java.nio.ByteBuffer;
import javax.swing.JFrame;
import plugin.ControllerPlugin;

/**
 *
 * @author Jason
 */
public class BasicKeyboardPlugin implements ControllerPlugin {
    
    public static final boolean DEBUG = false;
    
    private static final int NUMBER_OF_BUTTONS = 10;
    
    private static class Key {
        public int vkey;
        public int button;
    };
    
    private static Buttons controller1 = new Buttons();
    private static Key[] keys = new Key[NUMBER_OF_BUTTONS];
    private String name;
    
    /** Creates a new instance of Keyboard */
    public BasicKeyboardPlugin() {
        name = this.getClass().getName();
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") created.");
    }
    
    public void closePlugin() {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") closePlugin.");
    }
    
    public void controllerCommand(int control, ByteBuffer command) {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") controllerCommand.");
    }
    
    public void pluginAbout(JFrame hParent) {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") pluginAbout.");
    }
    
    public void pluginConfig(JFrame hParent) {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") pluginConfig.");
    }
    
    public void pluginTest(JFrame hParent) {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") pluginTest.");
    }
    
    public void getPluginInfo(PluginInfo pluginInfo) {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") getPluginInfo.");
        pluginInfo.version = 0x0100;
        pluginInfo.type = PLUGIN_TYPE_CONTROLLER;
        pluginInfo.name = "Basic KeyBoard plugin";
    }
    
    public void getKeys(int control, Buttons keys ) {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") getKeys.");
        if (keys == null)
            return;
        keys.value = controller1.value;
    }
    
    public void initiateControllers(JFrame hMainWindow, Control[] controls) {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") initiateControllers.");
        Buttons buttons = new Buttons();
        
        controls[0].present = true;
        controls[0].rawData = false;
        controls[0].plugin  = PLUGIN_NONE;
        controls[1].present = false;
        controls[2].present = false;
        controls[3].present = false;
        
        buttons.value  = 0;					//Up
        buttons.setXAxis((byte)80); // value may need to be reversed
        buttons.value |= Buttons.U_DPAD;
        keys[0] = new Key();
        keys[0].vkey   = KeyEvent.VK_UP;
        keys[0].button = buttons.value;
        
        buttons.value  = 0;					//Down
        buttons.setXAxis((byte)-80); // value may need to be reversed
        buttons.value |= Buttons.D_DPAD;
        keys[1] = new Key();
        keys[1].vkey   = KeyEvent.VK_DOWN;
        keys[1].button = buttons.value;
        
        buttons.value  = 0;					//left
        buttons.setYAxis((byte)-80); // value may need to be reversed
        buttons.value |= Buttons.L_DPAD;
        keys[2] = new Key();
        keys[2].vkey   = KeyEvent.VK_LEFT;
        keys[2].button = buttons.value;
        
        buttons.value  = 0;					//Right
        buttons.setYAxis((byte)80); // value may need to be reversed
        buttons.value |= Buttons.R_DPAD;
        keys[3] = new Key();
        keys[3].vkey   = KeyEvent.VK_RIGHT;
        keys[3].button = buttons.value;
        
        buttons.value  = 0;					//Start button
        buttons.value |= Buttons.START_BUTTON;
        keys[4] = new Key();
        keys[4].vkey   = KeyEvent.VK_ENTER;
        keys[4].button = buttons.value;
        
        buttons.value  = 0;					//A button
        buttons.value |= Buttons.A_BUTTON;
        keys[5] = new Key();
        keys[5].vkey   = 'X';
        keys[5].button = buttons.value;
        
        buttons.value  = 0;					//B button
        buttons.value |= Buttons.B_BUTTON;
        keys[6] = new Key();
        keys[6].vkey   = 'C';
        keys[6].button = buttons.value;
        
        // NEW
        
        buttons.value  = 0;					//L button
        buttons.value |= Buttons.L_TRIG;
        keys[7] = new Key();
        keys[7].vkey   = 'A';
        keys[7].button = buttons.value;
        
        buttons.value  = 0;					//R button
        buttons.value |= Buttons.R_TRIG;
        keys[8] = new Key();
        keys[8].vkey   = 'S';
        keys[8].button = buttons.value;
        
        buttons.value  = 0;					//Z button
        buttons.value |= Buttons.Z_TRIG;
        keys[9] = new Key();
        keys[9].vkey   = 'Z';
        keys[9].button = buttons.value;
    }
    
    public void readController(int control, ByteBuffer command ) {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") readController.");
    }
    
    public void romClosed() {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") romClosed.");
    }
    
    public void romOpen() {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") romOpen.");
    }
    
    public void wmKeyDown(int wParam, int lParam ) {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") wmKeyDown.");
        for (int count = 0; count < NUMBER_OF_BUTTONS; count++) {
            if (keys[count].vkey == wParam) {
                controller1.value |= keys[count].button;
                return;
            }
        }
    }
    
    public void wmKeyUp(int wParam, int lParam ) {
        if (DEBUG) System.out.println("Controller Plugin (" + name + ") wmKeyUp.");
        for (int count = 0; count < NUMBER_OF_BUTTONS; count++) {
            if (keys[count].vkey == wParam) {
                controller1.value &= ~keys[count].button;
                return;
            }
        }
    }
    
}
